#include "ZBFGApplication.h"

#include <fstream>
#include <cctype>
#include <cstring>
//#include <iostream>

static BPath BFG_start_path;
static bool check_file_scope(const char *);

//==== drill_through_directory (recursive function) ====================
bool drill_through_directory(const char *path) 
{ 
	bool string_found = false;
	int item_position;
	BStringItem *super_item;
	
	BDirectory dir(path); 
	
	if (dir.InitCheck() == B_OK)
	{ 
		BEntry item; 
		while((dir.GetNextEntry(&item) == B_OK) && search_in_progress_flag)
		{ 
			BPath workpath;
			item.GetPath(&workpath);
			if (item.IsDirectory() && zbfg_subdirectory_flag)
			{
				drill_through_directory(workpath.Path());
			}
			else
			{
				if (check_file_scope(workpath.Path()))
				{
					//
					strcpy(filename, workpath.Path());
				
					item_position = listwindow->CountItems();
					add_to_list(workpath.Path(), 0, true); 
					super_item = match_list[num_items_in_list-1]->bstring;
					listwindow->AddItem(super_item);
					if (BeGrep(super_item))
						string_found = true;
					else {
						remove_last_list_item();
						listwindow->RemoveItem(item_position);
					};
				} //file scope 
			} //item found 
		} //while
	}
			
	return (string_found); 
} 

//==== BeGrep ===========================================
bool BeGrep(BStringItem *super_item)
{
	int len;
	int line_count = 0;
	bool match_found = false;
				
	len = strlen(search_string);
	
	ifstream in(filename);  // open file
	
	while(in && search_in_progress_flag)
	{
		in.getline(file_buffer, FILEBUFFER_SIZE);
		if (in)
		{
			line_count++;
			
			register char *p = file_buffer;
			register int c = 0;
			register int temp;
			
			while (*p)
			{
				if (zbfg_ignore_case_flag)	//ignore lower case?
					*p=tolower(*p);	
				temp = *(search_string + c);
								
				if (*p == temp)
				{
					c++;
					if (c == len)  //match
					{
						//store match
						match_found = true;
						if (add_to_list(file_buffer, line_count, false)) //Item stored in linked list?
							listwindow->AddUnder(match_list[num_items_in_list - 1]->bstring, super_item); 
							//listwindow->AddItem(list_files->tail->bstring); 
					}
				}	
				else
					c = 0;
				p++;
			}
			
		}	
	}
	in.close();
	return match_found;
}

//=============================
static bool check_file_scope(const char *name)
{
	if (zbfg_limit_file_scope_flag)
	{
		if (file_scope_tokens == NULL || *file_scope_tokens == '\0') //special case when no tokens entered	
			return true;
			
		char *deliminators = " .-,;:*";
		char *buffer = new char [FILE_SCOPE_TOKENS_SIZE];
		if (buffer==NULL)
			return true; //since not enough memory, dont bias search with wrong results
		
		//strtok modifies arg1, store original
		char *buffer_pointer = buffer; 
		strcpy(buffer, file_scope_tokens);
					
		char *ret;
		ret = strrchr(name, '.'); //find last occurance of '.'
		if (ret==NULL || (ret++==NULL)) //no occurance or last character is '.'
		{
			delete [] buffer_pointer;
			return false;
		}
		
		buffer = strtok(buffer, deliminators);
		while (buffer)
		{
			if(strcmp(ret, buffer) == 0)
			{
				delete [] buffer_pointer;
				return true;
			}
			buffer=strtok(NULL, deliminators);
		}
		delete [] buffer_pointer;
		return false;	
	}
	else
		return true;
}

//=============================
void adjust_search_string_case(void)
{
	if (!zbfg_ignore_case_flag)	// do not change case when flag=false
		return;
	
	char *p = search_string;
	while (*p)
		*p = tolower(*p++);
	return;
}		

//=============================
// MAIN BFG loop - create as seperate thread and allow user to cancel
bool Start_BFG(const char *path)
{
	int32 handle_search_thread;
	
	adjust_search_string_case();
	delete_list();
	listwindow->EmptyList();
	
	BFG_start_path = path;

	search_in_progress_flag = true;
	handle_search_thread = spawn_thread(search_thread, "BeFileGrep Search Thread", B_NORMAL_PRIORITY, NULL);
	if (handle_search_thread) {
		resume_thread(handle_search_thread);
		return true;
	}
	else
		return false;	
}


int32 search_thread(void *dummy)
{
	drill_through_directory(BFG_start_path.Path());
	((ZApplication *) be_app)->fZWindowControl->PostMessage(SEARCH_FINISHED_MSG);
	return B_OK;
}